#include <cv.h>
#include <cvaux.h>
#include <highgui.h>
#include <stdio.h>
#include "U3Config.h"

IplImage *image = 0;
CvRect selection;
CvPoint origin;
int select_object = 0;
int track_object = 0;

void on_mouse( int event, int x, int y, int flags, void* param )
{
    if( !image ) return;

    if( image->origin ) y = image->height - y;

    if( select_object )
    {
        selection.x = MIN(x,origin.x);
        selection.y = MIN(y,origin.y);
        selection.width = selection.x + CV_IABS(x - origin.x);
        selection.height = selection.y + CV_IABS(y - origin.y);
        selection.x = MAX( selection.x, 0 );
        selection.y = MAX( selection.y, 0 );
        selection.width = MIN( selection.width, image->width );
        selection.height = MIN( selection.height, image->height );
        selection.width -= selection.x;
        selection.height -= selection.y;
    }

    switch( event )
    {
    case CV_EVENT_LBUTTONDOWN:
        origin = cvPoint(x,y);
        selection = cvRect(x,y,0,0);
        select_object = 1;
        break;
    case CV_EVENT_LBUTTONUP:
        select_object = 0;
        if( selection.width > 0 && selection.height > 0 )
        track_object = -1;
        break;
    }
}

int main()
{
	LJ_HANDLE lngHandle=0;
	unsigned int timer_duty_cycle_0 = 59536;
	unsigned int timer_duty_cycle_1 = 59536;
	unsigned int timer_clock_divisor = 1;
	
	CvCapture* capture=cvCaptureFromCAM(0);

    printf( "Tracking an object with a webcam fix onto a servo motor\n\n"
    "To initialize tracking, select the object with mouse in the center of the image\n\n"
	"Press ESC to quit the program\n\n");

    if(capture)
	{
  	    if(!U3Open(&lngHandle))
		{
			U3Reset(lngHandle);
			U3ConfigTimers(lngHandle, timer_clock_divisor, 59536);
			
			int c , barycentre_x, barycentre_y, vmin = 100, vmax = 256, smin = 128, hdims = 32;
			IplImage *frame = 0, *hsv = 0, *hue = 0, *mask = 0, *backproject = 0;

			CvHistogram *hist = 0;
			int backproject_mode = 0, lost = 0;
			CvRect track_window;
			CvBox2D track_box;
			CvConnectedComp track_comp;
			float hranges_arr[] = {0,180};
			float* hranges = hranges_arr;
			double diff_norm_x,diff_norm_y;

			frame=cvQueryFrame(capture);
			int width = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FRAME_WIDTH );
			int height= (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FRAME_HEIGHT);

			if( !image )
			{
				image = cvCreateImage( cvGetSize(frame), 8, 3 );
				image->origin = frame->origin;
				hsv = cvCreateImage( cvGetSize(frame), 8, 3 );
				hue = cvCreateImage( cvGetSize(frame), 8, 1 );
				mask = cvCreateImage( cvGetSize(frame), 8, 1 );
				backproject = cvCreateImage( cvGetSize(frame), 8, 1 );
				hist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );
				barycentre_x=width/2;barycentre_y=height/2;
			}

			cvNamedWindow("Tracking an object with a webcam fix onto a servo motor", 1);
			cvSetMouseCallback( "Tracking an object with a webcam fix onto a servo motor", on_mouse, 0 );

			while(1) 
			{
				frame=cvQueryFrame(capture);
				cvCopy( frame, image, 0 );
				cvCvtColor( image, hsv, CV_BGR2HSV );

				if( track_object )
				{
					int _vmin = vmin, _vmax = vmax;

					cvInRangeS( hsv, cvScalar(0,smin,MIN(_vmin,_vmax),0),
								cvScalar(180,256,MAX(_vmin,_vmax),0), mask );
					cvSplit( hsv, hue, 0, 0, 0 );

					if( track_object < 0 )
					{
						float max_val = 0.f;
						cvSetImageROI( hue, selection );
						cvSetImageROI( mask, selection );
						cvCalcHist( &hue, hist, 0, mask );
						cvGetMinMaxHistValue( hist, 0, &max_val, 0, 0 );
						cvConvertScale( hist->bins, hist->bins, max_val ? 255. / max_val : 0., 0 );
						cvResetImageROI( hue );
						cvResetImageROI( mask );
						track_window = selection;
						track_object = 1;
					}

					cvCalcBackProject( &hue, backproject, hist );
					cvAnd( backproject, mask, backproject, 0 );
					cvCamShift( backproject, track_window,
								cvTermCriteria( CV_TERMCRIT_EPS | CV_TERMCRIT_ITER, 10, 1 ),
								&track_comp, &track_box );

					if(track_box.size.height == 0.0)
					{
						barycentre_x=width/2;barycentre_y=height/2;
						if(!lost){printf(">>Tracker lost...\n");lost=1;}
					}
					else
					{
						track_window = track_comp.rect;
            			if( backproject_mode ) cvCvtColor( backproject, image, CV_GRAY2BGR );
						if( image->origin ) track_box.angle = -track_box.angle;
						cvEllipseBox( image, track_box, CV_RGB(255,0,0), 3, CV_AA, 0 );
						barycentre_x=track_window.x+track_window.width/2;
						barycentre_y=track_window.y+track_window.height/2;
						if(lost)printf(">>Tracker OK...\n");
						lost=0;
					}
				}
        
				if( select_object && selection.width > 0 && selection.height > 0 )
				{
					cvSetImageROI( image, selection );
					cvXorS( image, cvScalarAll(255), image, 0 );
					cvResetImageROI( image );
				}

				cvShowImage("Tracking an object with a webcam fix onto a servo motor", image );

				c = cvWaitKey(10);
				if( (char) c == 27 ){printf("\n");break;}
				switch( (char) c )
				{
				case 'b': backproject_mode ^= 1;break;
				case 'c': track_object = 0;	break;
				default:;
				}

				//printf("=%d %d %d %d\n",barycentre_x,barycentre_y,frame->width,frame->height);

				//Commande des servos
				if (track_object)
				{
					//servo PAN
					diff_norm_x = (barycentre_x-(frame->width)/2.0)/((frame->width)/2.0);
					if (fabs(diff_norm_x) >= 0.1)
						{
							timer_duty_cycle_0=(unsigned int) timer_duty_cycle_0-(768*diff_norm_x);
							if (timer_duty_cycle_0 > 61136){timer_duty_cycle_0=61136;printf(">>Servo end stop...\n");}
							if (timer_duty_cycle_0 < 57936){timer_duty_cycle_0=57936;printf(">>Servo end stop...\n");}
							U3ModifyDutyCycle(lngHandle,timer_duty_cycle_0,0);
						}
					//servo TILT
					diff_norm_y = (barycentre_y-(frame->height)/2.0)/((frame->height)/2.0);
					if (fabs(diff_norm_y) >= 0.1) 
						{
							timer_duty_cycle_1=(unsigned int) timer_duty_cycle_1-(368*diff_norm_y);
							if (timer_duty_cycle_1 > 61136){timer_duty_cycle_1=61136;printf(">>Servo end stop...\n");}
							if (timer_duty_cycle_1 < 57936){timer_duty_cycle_1=57936;printf(">>Servo end stop...\n");}
							U3ModifyDutyCycle(lngHandle,timer_duty_cycle_1,1);
						}
				}
				Sleep(50);
			}
		}
		else {printf("Labjack not found\n");}
	}
	else printf("Video stream not found\n\n");

	U3ConfigTimers(lngHandle, timer_clock_divisor, 59536);
	U3Close(&lngHandle);
    cvReleaseCapture( &capture );
    cvDestroyWindow("Capture Webcam");

	return 0;
}